;
; LCD driver example using PIC18F4220 and Seiko Epson model ECM-A0526 screen (640x480 monochrome)


	list	p=18f4550
	#include "p18f4550.inc"



; -----------------
; Port and bit definitions

#define	fonttable	hiragana
;#define	fonttable	highviz
;#define	fonttable	handscr

LCD			equ		PORTB
LCDTRIS		equ		TRISB
LCDC		equ		PORTA
LCDCTRIS	equ		TRISA

XCLK		equ	0
YCLK		equ	1
DIN			equ	2

; -----------------

	org	0
	goto	start



	org 200			; bootloader reloc. reset vector
	goto	start




	org H'220' 		; bootloader reloc. main

#include "string_m.inc"			; Strings to RS232 macro, feel free to remove if not needed.


start:

	bcf		WDTCON, SWDTEN		; kill watchdog timer

	movlw	B'11110000'			; INTOSC 8MHz, prefer external
	movwf	OSCCON				; You will want an external oscillator running as fast as possible; this chip supports 48MHz


	MOVLW 0Fh 					; Configure A/D
	MOVWF ADCON1 				; for digital inputs
	MOVLW 07h 					; Configure comparators
	MOVWF CMCON 				; for digital input


; --- init_ports
	clrf	LCDTRIS
	clrf	LCD		; 0s for data out

	bcf		LCDCTRIS, XCLK
	bcf		LCDCTRIS, YCLK
	bcf		LCDCTRIS, DIN
	clrf	LCDC
; ---

	pstrcpy	str_test,	SCREEN_START	; initialize character map memory with test message from program memory

	call	INITUSART

	movlw	UPPER	fonttable		; initialize upper byte of table ptr ... this chip doesn't count that high anyway...
	movwf	TBLPTRU

	sendstring	str_ready



main:

	setf	LCD			; dummy data
	call	lineclk		; initial line clock before data start
sendscreen:

	lfsr	FSR0, SCREEN_START	; point to start of RAM character table...
	clrf	CURLINE		; line counter for character generator

	bsf		LCDC, DIN	; first line has DIN sync for its entire duration...
	call	sendline	; ...
	bcf		LCDC, DIN	; after next line clock, but before 2nd line

	incf	CURLINE

	movlw	H'EF'		; 239 (0xEF) lines to be counted (does not include 1st line and 2 dummy lines at end)
	movwf	LINECNTL	; ...


sendscreen2:
	call	sendline
	incf	CURLINE

	decfsz	LINECNTL	; done sending lines?
	goto	sendscreen2	; if no

	call	senddummyline
	call	senddummyline

	goto	sendscreen

; -------------------

clk:

	bsf		LCDC, XCLK
	bcf		LCDC, XCLK
	return

lineclk:
	bsf		LCDC, YCLK

	;call	clk

	; --- inline 'clk'
	bsf		LCDC, XCLK
	bcf		LCDC, XCLK
	; --- end inline 'clk'

	bcf		LCDC, YCLK
	return


sendline:

	lfsr	FSR0, SCREEN_START	; HACK HACK HACK: Sending only 1st line over and over, rather than burn extra clocks figuring out where
								; in the ASCII memory map to start. (a.k.a. Lazy Tim) We'll worry about that when there's an SRAM to write to.

	movlw	H'4f'	;H'9F'				; (bytes/2) - 1 ...sending 2 bytes per loop (total 1 upper screen, 1 lower screen)
	movwf	BYTECNTL			; high not used
sendline2:

	movlw	HIGH	fonttable	; initialize TBLPTRx to start of font table
	movwf	TBLPTRH				; ...
	movlw	LOW		fonttable	;
	movwf	TBLPTRL				;

	movf	POSTINC0, w			; get current screen position's ASCII character
	mullw	H'10'				; *16d: convert to index into font table (in PRODH:PRODL)

	movf	PRODL, w			; add to font table base address
	addwf	TBLPTRL				;
	movf	PRODH, w			;
	addwfc	TBLPTRH				;

	movf	CURLINE, w			; get current line
	andlw	B'00001111'			; mod 16

	addwf	TBLPTRL				; add it to font table offset
								; NOTE: Should not be possible to rollover; low nibble should be 0 when we get here.
								; Align font table on a 256-byte boundary

	; Now the 8-bit row corresponding to the character's current row is pointed at by TBLPTRx

	TBLRD*						; get it in TABLAT
	movf	TABLAT, w
	swapf	WREG
	andlw	B'11110000'			; want high nibble only; the rest is for bottom screen
	movwf	LCD

;	call	clk
	; --- inline 'clk'
	bsf		LCDC, XCLK
	bcf		LCDC, XCLK
	; --- end inline 'clk'

	movf	TABLAT, w			; do it again for the next 4 bits
;	swapf	WREG
	andlw	B'11110000'
	movwf	LCD

	call	clk


	decfsz	BYTECNTL			; sent all bytes of line?
	goto	sendline2			; if no

	call	clk					; (n-1)th
	call	lineclk				; last byte of line is also a line clock

	return

senddummyline:

	movlw	H'9F'				; 160 bytes per line... 159 pulses the 'normal' way
	movwf	BYTECNTL			; high not used
senddummyline2:
	;call	clk

	movlw	H'FF'		; (match timing of 'call'...needed?)
	movwf	LCD

	nop
	nop
	nop
	nop


	bsf		LCDC, XCLK			; dummy byte clock
	bcf		LCDC, XCLK			;


	decfsz	BYTECNTL			; sent all bytes of line?
	goto	senddummyline2		; if no
								; last byte of line is also a line clock
	bsf		LCDC, YCLK

	bsf		LCDC, XCLK			; dummy byte clock
	bcf		LCDC, XCLK			;

	bcf		LCDC, YCLK

	return



#include	"4220uart.inc"
#include	"vars.inc"
	org 0x400
#include	"hiragana.inc"
;#include	"highviz.inc"
;#include	"handscr.inc"

; ------
str_ready:
	DW	"Hello World! Sending things to Earth....\r\n\x00"

str_test:
	DW	" The quick brown fox jumps over the lazy dog. AND THE LAZY DOG JUST SITS THERE.  \x00"

	end
